// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Default DOM parser for the Component Software Descriptor (CSD) XML DTD.</p>
 **/
public class CSDParser
{
    // csd 
    static final private String _class_name = "CSDParser";
    private org.w3c.dom.Document _doc;
    private String _uuid;           // the target implementation

    // public fields set during the parsing
    // and accessible from the outside
    public String descriptor_location;  // location of the CCD file for this implementation
    public String code_entrypoint;      // entrypoint of the code for this implementation
    public String code_location;        // location of the archive for this implementation
    public java.util.ArrayList valuetype_factories;

    // default constructor
    public
    CSDParser(org.w3c.dom.Document doc,
              String uuid)
    {
        // csd 
        _doc = doc;
        _uuid = uuid;

        // public fields
        descriptor_location = null;
        code_entrypoint = null;
        code_location = null;
        valuetype_factories = new java.util.ArrayList();
    }

    //
    //  internal operations
    //

    static private org.w3c.dom.Document
    openAsDocument(String fname, String dtdloc)
    {
        final String opname = "openAsDocument";
        // create SAX input source
        org.xml.sax.InputSource isource = null;
        try {
            isource = new org.xml.sax.InputSource(new java.io.FileInputStream(fname));
        } catch (java.io.FileNotFoundException ex) {
            final String msg = "FAILED (file not found: "+fname+")";
            TheLogger.error(_class_name, opname, msg, ex);
            return null;
        }

        // change location of DTD in document
        String loc = isource.getSystemId();
        isource.setSystemId(dtdloc+"/"+loc);

        // obtain new DOM parser factory
        // NOTE: disable namespaces and validation
        javax.xml.parsers.DocumentBuilderFactory fact = javax.xml.parsers.DocumentBuilderFactory.newInstance();
        fact.setNamespaceAware(false);
        fact.setValidating(false);

        // create DOM parser and parse XML file
        try {
            javax.xml.parsers.DocumentBuilder parser = fact.newDocumentBuilder();
            return parser.parse(isource);
        } catch (Exception ex) {
            TheLogger.error(_class_name, opname, "FAILED", ex);
            return null;
        }
    }    

    final private void
    parseDocument()
    {
        org.w3c.dom.Element root = _doc.getDocumentElement();
        org.w3c.dom.NodeList childs = root.getChildNodes();
        org.w3c.dom.Node node = null;
        for (int i=0;i<childs.getLength();i++) {
            node = childs.item(i);
            switch (node.getNodeType()) {
            case org.w3c.dom.Node.ELEMENT_NODE: 
                org.w3c.dom.Element element = (org.w3c.dom.Element)node;
                String tagname = element.getTagName();

                if (tagname.equals("implementation")) {
                    // equiv XPath: "softpkg/implementation[@id=uuid]"
                    String id = element.getAttribute("id");
                    if (_uuid.equals(id)) {
                        parseImplementation(element);
                    }
                }

                break;
            default:
                break;
            }
        }
    }

    final private void
    parseImplementation(org.w3c.dom.Element impl)
    {
        final String opname = "parseImplementation";

        // code
        // equiv XPath: "softpkg/implementation[@id=uuid]/code"
        org.w3c.dom.NodeList codes = impl.getElementsByTagName("code");
        if (codes.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing code element, UUID="+_uuid+")");
        }

        // NOTE: there's only one code element
        org.w3c.dom.Element code = (org.w3c.dom.Element)codes.item(0);
        parseCode(code);

        // dependencies
        // equiv XPath: "softpkg/implementation[@id=uuid]/dependency"
        org.w3c.dom.NodeList dependencies = impl.getElementsByTagName("dependency");
        org.w3c.dom.Element dep = null;
        for (int i=0;i<dependencies.getLength();i++) {
            dep = (org.w3c.dom.Element)dependencies.item(i);
            parseDependency(dep);
        }

        // descriptor
        // equiv XPath: "softpkg/implementation[@id=uuid]/descriptor"
        org.w3c.dom.NodeList descriptors = impl.getElementsByTagName("descriptor");
        if (descriptors.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing descriptor element, UUID="+_uuid+")");
        }

        // NOTE: there's only one descriptor element
        org.w3c.dom.Element desc = (org.w3c.dom.Element)descriptors.item(0);
        parseDescriptor(desc);
    }

    final private void
    parseCode(org.w3c.dom.Element code)
    {
        final String opname = "parseCode";

        // entrypoint
        // equiv XPath: "softpkg/implementation[@id=uuid]/code/entrypoint"
        org.w3c.dom.NodeList entrypoints = code.getElementsByTagName("entrypoint");
        if (entrypoints.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing entrypoint element, UUID="+_uuid+")");
        }

        // NOTE: there's only one entrypoint element
        org.w3c.dom.Element entrypoint = (org.w3c.dom.Element)entrypoints.item(0);
        org.w3c.dom.NodeList datas = entrypoint.getChildNodes();
        for (int i=0;i<datas.getLength();i++) {
            org.w3c.dom.Node node = datas.item(i);
            if ((node.getNodeName().equals("#cdata-section")) ||
                (node.getNodeName().equals("#text"))) {
                org.w3c.dom.CharacterData data = (org.w3c.dom.CharacterData)node;
                code_entrypoint = data.getData();
            }
        }

        if (code_entrypoint==null) {
            TheLogger.error(_class_name, opname, "FAILED (missing entrypoint value, UUID="+_uuid+")");
        }

        // code location
        // NOTE: only support fileinarchive
        // equiv XPath: "softpkg/implementation[@id=uuid]/code/fileinarchive"
        org.w3c.dom.NodeList fileinarchives = code.getElementsByTagName("fileinarchive");
        if (fileinarchives.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing fileinarchive element, UUID="+_uuid+")");
        }

        // NOTE: there's only one fileinarchive element
        org.w3c.dom.Element fileinarchive = (org.w3c.dom.Element)fileinarchives.item(0);
        // equiv XPath: "softpkg/implementation[@id=uuid]/code/fileinarchive[@name]"
        code_location = fileinarchive.getAttribute("name");
    }

    final private void
    parseDependency(org.w3c.dom.Element dep)
    {
        final String opname = "parseDependency";

        // NOTE: only parse valuetypefactory dependencies
        // valuetypefactory
        // equiv XPath: "softpkg/implementation[@id=uuid]/dependency/valuetypefactory"
        org.w3c.dom.NodeList valuetypefactorys = dep.getElementsByTagName("valuetypefactory");
        if (valuetypefactorys.getLength()==0) {
            return ;
        }

        // NOTE: there's only one valuetypefactory element
        org.w3c.dom.Element valuetypefactory = (org.w3c.dom.Element)valuetypefactorys.item(0);
        org.coach.ECM.ValuetypeFactoryDependency vfdep = new org.coach.ECM.ValuetypeFactoryDependency();
        vfdep.repid = valuetypefactory.getAttribute("repid");
        vfdep.value_entrypoint = valuetypefactory.getAttribute("valueentrypoint");
        vfdep.factory_entrypoint = valuetypefactory.getAttribute("factoryentrypoint");

        // code location
        // NOTE: only support fileinarchive
        // equiv XPath: "softpkg/implementation[@id=uuid]/dependency/valuetypefactory/fileinarchive"
        org.w3c.dom.NodeList fileinarchives = valuetypefactory.getElementsByTagName("fileinarchive");
        if (fileinarchives.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing fileinarchive element, UUID="+_uuid+")");
        }

        // NOTE: there's only one fileinarchive element
        org.w3c.dom.Element fileinarchive = (org.w3c.dom.Element)fileinarchives.item(0);
        vfdep.code_location = fileinarchive.getAttribute("name");

        // store
        valuetype_factories.add(vfdep);
    }


    final private void
    parseDescriptor(org.w3c.dom.Element desc)
    {
        final String opname = "parseDescriptor";

        // descriptor location
        // NOTE: only support fileinarchive
        // equiv XPath req: "softpkg/implementation[@id=_uuid]/descriptor/fileinarchive/@name"
        org.w3c.dom.NodeList fileinarchives = desc.getElementsByTagName("fileinarchive");
        if (fileinarchives.getLength()==0) {
            TheLogger.error(_class_name, opname, "FAILED (missing fileinarchive element, UUID="+_uuid+")");
        }

        // NOTE: there's only one fileinarchive element
        org.w3c.dom.Element fileinarchive = (org.w3c.dom.Element)fileinarchives.item(0);
        descriptor_location = fileinarchive.getAttribute("name");
    }

    //
    //  static public operations
    //

    static public CSDParser
    parse(String fname, String dtdloc, String uuid)
    {
        CSDParser parser = null;
        try {
            org.w3c.dom.Document doc = openAsDocument(fname, dtdloc);
            parser = new CSDParser(doc, uuid);
            parser.parseDocument();
        } catch (Exception ex) {
            final String opname = "parse";
            TheLogger.error(_class_name, opname, "FAILED", ex);
            return null;
        }

        return parser;
    }
}
