// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.runtime;

/**
 **  <p>Default implementation of the <tt>ORBFactory</tt> factory.</p>
 **/
public class ORBFactoryImpl
extends org.omg.CORBA.LocalObject
implements ORBFactory
{
    // system factory
    static final private String _class_name = "ORBFactoryImpl";
    private java.util.ArrayList _default_factories;
    private java.util.ArrayList _default_initializers;

    // default constructor
    protected
    ORBFactoryImpl()
    {
        // system factory
        _default_factories = new java.util.ArrayList();
        _default_initializers = new java.util.ArrayList();
    }

    //
    //  entry point
    //

    static public SystemFactory
    create_factory()
    {
        return new ORBFactoryImpl();
    }

    //
    //  internal operations
    //

    // NOTE: this operation shall take into account the specifics of each
    // ORB implementation

    static private org.omg.CORBA.ORB
    initializeORB(String[] params, java.util.Properties props)
    {
        return org.omg.CORBA.ORB.init(params, props);

    }

    // NOTE: extracts an IOR from an URN
    static protected String
    extractIOR(String name)
    {
        final String opname = "extractIOR";

        // check if already an IOR
        if (name.startsWith("IOR:")) {
            return name;
        }

        // try to open as local file
        java.io.InputStream in = null;
        try {
            in = new java.io.FileInputStream(name);
        } catch (java.io.FileNotFoundException ex) {
            // ignore
        }

        if (in==null) {
            // try to open as URL
            java.net.URL url = null;
            try {
                url = new java.net.URL(name);
                in = url.openStream();
            } catch (Exception ex) {
                // ignore
            }
        }

        if (in==null) {
            final String msg = "FAILED (invalid format: "+name+")";
            TheLogger.error(_class_name, opname, msg);
            return null;
        }

        // read reference 
        java.io.BufferedReader reader = null;
        StringBuffer res = new StringBuffer();

        try {
            reader = new java.io.BufferedReader(new java.io.InputStreamReader(in));
            while(reader.ready()) {
                res.append(reader.readLine());
            }

            reader.close();
        } catch(Exception ex) {
            final String msg = "IGNORE (resource: "+name+")";
            TheLogger.error(_class_name, opname, msg, ex);
            return null;
        }

        return res.toString();
    }

    // NOTE: creates a new ORB instance, activates root POA and registers it to the main thread
    // ORB creation takes into account:
    //   - ORB initializers
    //   - stringified initial references (passed as command line arguments)
    //   - value factories (register using CORBA 2.3 ORB register_value_factory operation)
    static private org.omg.CORBA.ORB
    createORB(ORBConfiguration cfg)
    {
        final String opname = "createORB";

        // create ORB
        org.omg.CORBA.ORB orb = null;

        // add ORBInitializer entry points to properties (Java mapping)
        String[] orb_inits = cfg.orb_initializers();
        java.util.Properties props = System.getProperties();
        for (int i=0;i<orb_inits.length;i++) {
            TheLogger.debug(_class_name, "createORB", "ORBInitializer registered: "+orb_inits[i]);
            props.put("org.omg.PortableInterceptor.ORBInitializerClass."+orb_inits[i], "");
        }

        // add stringified initial references as arguments
        StringifiedInitialReference[] strirefs = cfg.stringified_initial_references();
        String[] args = new String[strirefs.length*2];
        int j=0;
        for (int i=0;i<strirefs.length;i++) {
            //TheLogger.debug(_class_name, opname, "Arg #"+j+": -ORBInitRef "+strirefs[i].name+"="+strirefs[i].ref);
            args[j] = "-ORBInitRef";
            args[j+1] = strirefs[i].name+"="+strirefs[i].ref;

            // check if ref is an URL (as OpenORB does not manage this correctly apparently)
            //if (strirefs[i].ref.startsWith("file:")) {
            //    args[j+1] = strirefs[i].name+"="+extractIORFromFile(strirefs[i].ref);
            //}
            //else {
            //    args[j+1] = strirefs[i].name+"="+strirefs[i].ref;
            //}

            j=j+2;
        }

        // do NOT add initial references
        // NOTE: non stringified initial references are added in the ORBService object
        // (and not in the underlying ORB instance as it requires a CORBA 2.5 ORB for that)

        String cline = "";
        for (int i=0;i<args.length;i++) {
            cline = cline+" "+args[i];
        }

        //TheLogger.debug(_class_name, opname, "command line: "+cline);
        orb = initializeORB(args, props);

        // activate root POA
        try {
            org.omg.PortableServer.POA rootPOA = null;
            org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
            rootPOA = org.omg.PortableServer.POAHelper.narrow(obj);
            rootPOA.the_POAManager().activate();
        } catch (Exception ex) {
            TheLogger.error(_class_name, opname, "FAILED", ex);
            return null;
        }

        // run the ORB in the main thread
        Runtime.getMainThread().runORB(orb);

        // add value factories
        // NOTE: requires a CORBA 2.3 ORB
        ValueFactory[] vfacts = cfg.value_factories();
        org.omg.CORBA_2_3.ORB orb23 = (org.omg.CORBA_2_3.ORB)orb;
        org.omg.CORBA.portable.ValueFactory vfact = null;

        for (int i=0;i<vfacts.length;i++) {
            //TheLogger.debug(_class_name, opname, "Register ValueFactory: "+vfacts[i].type_id());
            vfact = vfacts[i].value_factory();
            orb23.register_value_factory(vfacts[i].type_id(), vfact);
        }

        return orb;
    }

    private ORBConfiguration
    addDefaults(ORBConfiguration cfg)
    {
        // value factories
        ValueFactory[] ivfacts = cfg.value_factories();
        ValueFactory[] nfacts = null;
        // add to default ones
        if (_default_factories.size()==0) {
            nfacts = ivfacts;
        }
        else {
            java.util.ArrayList dvfacts = _default_factories;
            java.util.ArrayList nvfacts = new java.util.ArrayList(dvfacts);
            nvfacts.addAll(java.util.Arrays.asList(ivfacts));
            nfacts = (ValueFactory[])nvfacts.toArray(new ValueFactory[0]);
        }

        // ORB initializers
        String[] inits = cfg.orb_initializers();
        String[] ninits = null;
        // add default ones
        if (_default_initializers.size()==0) {
            ninits = inits;
        }
        else {
            java.util.ArrayList vinits = _default_initializers;
            java.util.ArrayList nvinits = new java.util.ArrayList(vinits);
            nvinits.addAll(java.util.Arrays.asList(inits));
            ninits = (String[])nvinits.toArray(new String[0]);
        }

        // return new configuration
        return new ORBConfigurationImpl(ninits,
                                        nfacts,
                                        cfg.initial_references(),
                                        cfg.stringified_initial_references());
    }

    //
    //  IDL:objectweb.org/corba/runtime/SystemFactory:1.0
    //

    final public SystemComponent
    create_system_component(SystemConfiguration cfg)
    {
        ORBConfiguration orbcfg = (ORBConfiguration)cfg;

        // add default value factories / ORB initializer to init info
        ORBConfiguration neworbcfg = addDefaults(orbcfg);

        // create new ORB instance
        org.omg.CORBA.ORB orb = createORB(neworbcfg);

        // create a new ORB service instance
        ORBServiceImpl orbs = new ORBServiceImpl(orb);

        // complete configuration
        orbs.system_configuration_complete(neworbcfg);

        // set initial references as they are not set on the ORB instance
        InitialReference[] refs = neworbcfg.initial_references();
        for (int i=0;i<refs.length;i++) {
            orbs.register_initial_references(refs[i].name, refs[i].ref);
        }

        return orbs;
    }

    //
    //  IDL:objectweb.org/corba/runtime/ORBFactory:1.0
    //

    final public void
    add_default_valuefactory(ValueFactory vfact) 
    {
        _default_factories.add(vfact);
    }

    final public void
    add_default_initializer(String orbinit)
    {
        _default_initializers.add(orbinit);
    }
}
